@@ -1,8 +1,9 @@
-/* A Bison parser, made by GNU Bison 3.0.  */
+/* A Bison parser, made by GNU Bison 3.8.2.  */
 
 /* Bison interface for Yacc-like parsers in C
 
-   Copyright (C) 1984, 1989-1990, 2000-2013 Free Software Foundation, Inc.
+   Copyright (C) 1984, 1989-1990, 2000-2015, 2018-2021 Free Software Foundation,
+   Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,7 +16,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 /* As a special exception, you may create a larger work that contains
    part or all of the Bison parser skeleton and distribute that work
@@ -30,6 +31,10 @@
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
+/* DO NOT RELY ON FEATURES THAT ARE NOT DOCUMENTED in the manual,
+   especially those whose name start with YY_ or yy_.  They are
+   private implementation details that can be changed or removed.  */
+
 #ifndef YY_YY_DEFPARSE_H_INCLUDED
 # define YY_YY_DEFPARSE_H_INCLUDED
 /* Debug traces.  */
@@ -40,43 +45,52 @@
 extern int yydebug;
 #endif
 
-/* Token type.  */
+/* Token kinds.  */
 #ifndef YYTOKENTYPE
 # define YYTOKENTYPE
   enum yytokentype
   {
-    NAME = 258,
-    LIBRARY = 259,
-    DESCRIPTION = 260,
-    STACKSIZE = 261,
-    HEAPSIZE = 262,
-    CODE = 263,
-    DATA = 264,
-    SECTIONS = 265,
-    EXPORTS = 266,
-    IMPORTS = 267,
-    VERSIONK = 268,
-    BASE = 269,
-    CONSTANT = 270,
-    READ = 271,
-    WRITE = 272,
-    EXECUTE = 273,
-    SHARED = 274,
-    NONSHARED = 275,
-    NONAME = 276,
-    PRIVATE = 277,
-    SINGLE = 278,
-    MULTIPLE = 279,
-    INITINSTANCE = 280,
-    INITGLOBAL = 281,
-    TERMINSTANCE = 282,
-    TERMGLOBAL = 283,
-    EQUAL = 284,
-    ID = 285,
-    NUMBER = 286
+    YYEMPTY = -2,
+    YYEOF = 0,                     /* "end of file"  */
+    YYerror = 256,                 /* error  */
+    YYUNDEF = 257,                 /* "invalid token"  */
+    NAME = 258,                    /* NAME  */
+    LIBRARY = 259,                 /* LIBRARY  */
+    DESCRIPTION = 260,             /* DESCRIPTION  */
+    STACKSIZE = 261,               /* STACKSIZE  */
+    HEAPSIZE = 262,                /* HEAPSIZE  */
+    CODE = 263,                    /* CODE  */
+    DATA = 264,                    /* DATA  */
+    SECTIONS = 265,                /* SECTIONS  */
+    EXPORTS = 266,                 /* EXPORTS  */
+    IMPORTS = 267,                 /* IMPORTS  */
+    VERSIONK = 268,                /* VERSIONK  */
+    BASE = 269,                    /* BASE  */
+    CONSTANT = 270,                /* CONSTANT  */
+    READ = 271,                    /* READ  */
+    WRITE = 272,                   /* WRITE  */
+    EXECUTE = 273,                 /* EXECUTE  */
+    SHARED = 274,                  /* SHARED  */
+    NONSHARED = 275,               /* NONSHARED  */
+    NONAME = 276,                  /* NONAME  */
+    PRIVATE = 277,                 /* PRIVATE  */
+    SINGLE = 278,                  /* SINGLE  */
+    MULTIPLE = 279,                /* MULTIPLE  */
+    INITINSTANCE = 280,            /* INITINSTANCE  */
+    INITGLOBAL = 281,              /* INITGLOBAL  */
+    TERMINSTANCE = 282,            /* TERMINSTANCE  */
+    TERMGLOBAL = 283,              /* TERMGLOBAL  */
+    EQUAL = 284,                   /* EQUAL  */
+    ID = 285,                      /* ID  */
+    NUMBER = 286                   /* NUMBER  */
   };
+  typedef enum yytokentype yytoken_kind_t;
 #endif
-/* Tokens.  */
+/* Token kinds.  */
+#define YYEMPTY -2
+#define YYEOF 0
+#define YYerror 256
+#define YYUNDEF 257
 #define NAME 258
 #define LIBRARY 259
 #define DESCRIPTION 260
@@ -109,17 +123,18 @@ extern int yydebug;
 
 /* Value type.  */
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
-typedef union YYSTYPE YYSTYPE;
 union YYSTYPE
 {
-#line 28 "defparse.y" /* yacc.c:1915  */
+#line 28 "defparse.y"
 
   char *id;
   const char *id_const;
   int number;
 
-#line 122 "defparse.h" /* yacc.c:1915  */
+#line 135 "defparse.h"
+
 };
+typedef union YYSTYPE YYSTYPE;
 # define YYSTYPE_IS_TRIVIAL 1
 # define YYSTYPE_IS_DECLARED 1
 #endif
@@ -127,6 +142,8 @@ union YYSTYPE
 
 extern YYSTYPE yylval;
 
+
 int yyparse (void);
 
+
 #endif /* !YY_YY_DEFPARSE_H_INCLUDED  */
