@@ -1,5 +1,5 @@
 /* coff object file format
-   Copyright (C) 1989-2023 Free Software Foundation, Inc.
+   Copyright (C) 1989-2025 Free Software Foundation, Inc.
 
    This file is part of GAS.
 
@@ -1150,8 +1150,8 @@ obj_coff_weak (int ignore ATTRIBUTE_UNUSED)
 	}
       c = 0;
       symbolP = symbol_find_or_make (name);
-      *input_line_pointer = c;
-      SKIP_WHITESPACE_AFTER_NAME ();
+      restore_line_pointer (c);
+      SKIP_WHITESPACE ();
       S_SET_WEAK (symbolP);
 
       if (c == ',')
@@ -1177,6 +1177,15 @@ coff_obj_read_begin_hook (void)
   tag_init ();
 }
 
+void
+coff_assign_symbol (symbolS *symp ATTRIBUTE_UNUSED)
+{
+#ifndef TE_PE
+  /* "set" symbols are local unless otherwise specified.  */
+  SF_SET_LOCAL (symp);
+#endif
+}
+
 symbolS *coff_last_function;
 #ifndef OBJ_XCOFF
 static symbolS *coff_last_bf;
@@ -1548,8 +1557,8 @@ obj_coff_section (int ignore ATTRIBUTE_UNUSED)
 
   c = get_symbol_name (&section_name);
   name = xmemdup0 (section_name, input_line_pointer - section_name);
-  *input_line_pointer = c;
-  SKIP_WHITESPACE_AFTER_NAME ();
+  restore_line_pointer (c);
+  SKIP_WHITESPACE ();
 
   exp = 0;
   flags = SEC_NO_FLAGS;
@@ -1804,11 +1813,10 @@ coff_frob_section (segT sec)
 }
 
 void
-obj_coff_init_stab_section (segT seg)
+obj_coff_init_stab_section (segT stab ATTRIBUTE_UNUSED, segT stabstr)
 {
   const char *file;
   char *p;
-  char *stabstr_name;
   unsigned int stroff;
 
   /* Make space for this first symbol.  */
@@ -1816,8 +1824,7 @@ obj_coff_init_stab_section (segT seg)
   /* Zero it out.  */
   memset (p, 0, 12);
   file = as_where ((unsigned int *) NULL);
-  stabstr_name = concat (seg->name, "str", (char *) NULL);
-  stroff = get_stab_string_offset (file, stabstr_name, true);
+  stroff = get_stab_string_offset (file, stabstr);
   know (stroff == 1);
   md_number_to_chars (p, stroff, 4);
 }
@@ -1849,7 +1856,7 @@ symbol_dump (void)
 
 #endif /* DEBUG */
 
-const pseudo_typeS coff_pseudo_table[] =
+static const pseudo_typeS coff_pseudo_table[] =
 {
   {"ABORT", s_abort, 0},
   /* We accept the .bss directive for backward compatibility with
@@ -1890,14 +1897,14 @@ const pseudo_typeS coff_pseudo_table[] =
 };
 
 
-/* Support for a COFF emulation.  */
-
-static void
+void
 coff_pop_insert (void)
 {
   pop_insert (coff_pseudo_table);
 }
 
+#ifdef USE_EMULATIONS /* Support for a COFF emulation.  */
+
 static int
 coff_separate_stab_sections (void)
 {
@@ -1912,6 +1919,7 @@ const struct format_ops coff_format_ops =
   0,    /* begin */
   0,	/* end.  */
   c_dot_file_symbol,
+  coff_assign_symbol,
   coff_frob_symbol,
   0,	/* frob_file */
   0,	/* frob_file_before_adjust */
@@ -1928,7 +1936,6 @@ const struct format_ops coff_format_ops =
   0,	/* s_get_type */
   0,	/* s_set_type */
   0,	/* copy_symbol_attributes */
-  0,	/* generate_asm_lineno */
   0,	/* process_stab */
   coff_separate_stab_sections,
   obj_coff_init_stab_section,
@@ -1940,3 +1947,5 @@ const struct format_ops coff_format_ops =
   coff_obj_symbol_clone_hook,
   coff_adjust_symtab
 };
+
+#endif /* USE_EMULATIONS */
