@@ -1,5 +1,5 @@
 /* expr.h -> header file for expr.c
-   Copyright (C) 1987-2022 Free Software Foundation, Inc.
+   Copyright (C) 1987-2025 Free Software Foundation, Inc.
 
    This file is part of GAS, the GNU Assembler.
 
@@ -153,9 +153,12 @@ enum expr_mode
 {
   expr_evaluate,
   expr_normal,
-  expr_defer
+  expr_defer,
+  expr_defer_incl_dot,
 };
 
+#define expr_defer_p(m) ((m) >= expr_defer)
+
 /* "result" should be type (expressionS *).  */
 #define expression(result) expr (0, result, expr_normal)
 #define expression_and_evaluate(result) expr (0, result, expr_evaluate)
@@ -175,19 +178,21 @@ typedef char operator_rankT;
 extern char get_symbol_name (char **);
 extern char restore_line_pointer (char);
 extern void expr_begin (void);
+extern void expr_end (void);
 extern void expr_set_precedence (void);
 extern void expr_set_rank (operatorT, operator_rankT);
 extern void add_to_result (expressionS *, offsetT, int);
 extern void subtract_from_result (expressionS *, offsetT, int);
 extern segT expr (int, expressionS *, enum expr_mode);
 extern unsigned int get_single_number (void);
-extern symbolS *make_expr_symbol (expressionS * expressionP);
+extern symbolS *make_expr_symbol (const expressionS * expressionP);
 extern int expr_symbol_where (symbolS *, const char **, unsigned int *);
-extern void current_location (expressionS *);
+extern void current_location (expressionS *, enum expr_mode);
 extern symbolS *expr_build_uconstant (offsetT);
 extern symbolS *expr_build_dot (void);
 extern uint32_t generic_bignum_to_int32 (void);
 extern uint64_t generic_bignum_to_int64 (void);
 extern int resolve_expression (expressionS *);
+extern void resolve_register (expressionS *);
 
 extern bool literal_prefix_dollar_hex;
